/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "common.h"
#include "userinfo.h"

#include <stdlib.h>

UserInfoModule *userinfo;

void userinfo_init(void)
{
	const char *name, *args;

	userinfo = NULL;

	name = getenv("USERINFO");
	if (name == NULL)
		i_fatal("USERINFO environment is unset");

#ifdef USERINFO_PASSWD
	if (strcasecmp(name, "passwd") == 0)
		userinfo = &userinfo_passwd;
#endif
#ifdef USERINFO_SHADOW
	if (strcasecmp(name, "shadow") == 0)
		userinfo = &userinfo_shadow;
#endif
#ifdef USERINFO_PAM
	if (strcasecmp(name, "pam") == 0)
		userinfo = &userinfo_pam;
#endif

	if (userinfo == NULL)
		i_fatal("Unknown userinfo type '%s'", name);

	/* initialize */
	if (userinfo->init != NULL) {
		args = getenv("USERINFO_ARGS");
		if (args == NULL) args = "";

		userinfo->init(args);
	}
}

void userinfo_deinit(void)
{
	if (userinfo != NULL && userinfo->deinit != NULL)
		userinfo->deinit();
}
