/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"
#include "mmap-util.h"
#include "maildir-index.h"
#include "mail-index-util.h"

int maildir_record_update(MailIndex *index, MailIndexRecord *rec,
			  MailIndexUpdate *update, int fd, const char *path)
{
	void *mmap_base;
	size_t mmap_length;

	i_assert(path != NULL);

	/* we need only the header which probably fits into one page,
	   so don't use MADV_SEQUENTIAL which would just read more than
	   is needed. */
	mmap_base = mmap_ro_file(fd, &mmap_length);
	if (mmap_base == MAP_FAILED) {
		index_set_error(index, "update: mmap() failed with file %s: %s",
				path, strerror(errno));
		return FALSE;
	}

	if (mmap_base == NULL) {
		/* empty file */
		return TRUE;
	}

	mail_index_update_headers(update, mmap_base, mmap_length, NULL, NULL);
	(void)munmap(mmap_base, mmap_length);
	return TRUE;
}
