/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"
#include "rfc822-tokenize.h"
#include "message-content-parser.h"

Rfc822Token *message_content_parse_header(Pool pool, const char *value,
					  ParseContentFunc func,
					  ParseContentParamFunc param_func,
					  void *user_data)
{
	Rfc822Token *tokens;
	int i, next, ntokens;

	tokens = rfc822_tokenize(pool, value, &ntokens, NULL, NULL);
	if (tokens == NULL) {
		/* error */
		return NULL;
	}

	/* first ';' separates the parameters */
	for (i = 0; i < ntokens; i++) {
		if (tokens[i].token == ';')
			break;
	}

	if (func != NULL)
		func(tokens, i, user_data);

	if (param_func != NULL) {
		/* parse the parameters */
		for (i++; i < ntokens; i = next) {
			/* find the next ';' */
			for (next = i+1; next < ntokens; next++) {
				if (tokens[next].token == ';')
					break;
			}

			if (i+2 < next &&
			    tokens[i].token == 'A' &&
			    tokens[i+1].token == '=') {
				/* <atom> = <value> */
				param_func(tokens + i, tokens + i + 2,
					   next - (i+2), user_data);
			}
		}
	}

	return tokens;
}
