/*
    Copyright (C) 2002 Timo Sirainen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "lib.h"
#include "settings.h"

#include <unistd.h>
#include <pwd.h>
#include <sys/stat.h>

/* login */
//const char *set_login_executable = PKG_LIBDIR "/imap-login";
const char *set_login_executable = "../login/imap-login";
const char *set_login_user = "imapd";
const char *set_login_chroot = PKG_RUNDIR;

uid_t set_login_uid;
gid_t set_login_gid;

int set_login_processes_count = 1;

/* imap */
//const char *set_imap_executable = PKG_LIBDIR "/imap";
const char *set_imap_executable = "../imap/imap";
const char *set_valid_chroot_dirs = "";

int set_first_valid_uid = 500, set_last_valid_uid = 0;
int set_first_valid_gid = 500, set_last_valid_gid = 0;

/* auth */
AuthConfig *auth_processes_config = NULL;

static void get_login_uid(void)
{
	struct passwd *pw;

	if ((pw = getpwnam(set_login_user)) == NULL)
		i_fatal("Login user doesn't exist: %s", set_login_user);

	set_login_uid = pw->pw_uid;
	set_login_gid = pw->pw_gid;
}

static AuthConfig *auth_config_new(const char *name)
{
	AuthConfig *auth;

	auth = i_new(AuthConfig, 1);
	auth->name = i_strdup(name);

	auth->next = auth_processes_config;
        auth_processes_config = auth;
	return auth;
}

static void auth_config_read(void)
{
	AuthConfig *auth;

	auth = auth_config_new("auth");
	auth->methods = "plain";
	auth->userinfo = "shadow";
	//auth->executable = PKG_LIBDIR "/imap-auth";
	auth->executable = "../auth/imap-auth";
	auth->user = "root";
	auth->count = 1;
	auth->chroot = NULL;
}

void settings_read(void)
{
	if (access(set_login_executable, X_OK) == -1) {
		i_fatal("Can't use login executable %s: %s",
			set_login_executable, strerror(errno));
	}

	if (access(set_imap_executable, X_OK) == -1) {
		i_fatal("Can't use imap executable %s: %s",
			set_imap_executable, strerror(errno));
	}

	get_login_uid();
	auth_config_read();

	/* since it's under /var/run by default, it may have been deleted */
	if (mkdir(set_login_chroot, 0700) == 0)
		(void)chown(set_login_chroot, set_login_uid, set_login_gid);
	if (access(set_login_chroot, X_OK) == -1) {
		i_fatal("Can't access login directory %s: %s",
			set_login_chroot, strerror(errno));
	}

	if (set_login_processes_count < 1)
		i_fatal("login_processes_count must be at least 1");
	if (set_first_valid_uid < set_last_valid_uid)
		i_fatal("first_valid_uid can't be larger than last_valid_uid");
	if (set_first_valid_gid < set_last_valid_gid)
		i_fatal("first_valid_gid can't be larger than last_valid_gid");
}
